﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 커스텀 색상
    /// </summary>
    [TypeConverter(typeof(ExpandableObjectConverter))]
    public class CustomColor
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 커스텀 색상 변경시 이벤트 - CustomColorChanged

        /// <summary>
        /// 커스텀 색상 변경시 이벤트
        /// </summary>
        public event EventHandler<EventArgs> CustomColorChanged;

        #endregion
        
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region FieldsPrivate

        /// <summary>
        /// 테두리 색상
        /// </summary>
        private Color borderColor = ProfessionalColors.GripDark;

        /// <summary>
        /// 내부 테두리 색상
        /// </summary>
        private Color innerBorderColor = ProfessionalColors.GripLight;

        /// <summary>
        /// 제목 바 시작 색상
        /// </summary>
        private Color captionBarBeginColor = ProfessionalColors.ToolStripGradientBegin;

        /// <summary>
        /// 제목 바 중간 색상
        /// </summary>
        private Color captionBarMiddleColor = ProfessionalColors.ToolStripGradientMiddle;

        /// <summary>
        /// 제목 바 종료 색상
        /// </summary>
        private Color captionBarEndColor = ProfessionalColors.ToolStripGradientEnd;

        /// <summary>
        /// 제목 텍스트
        /// </summary>
        private Color captionText = SystemColors.ControlText;

        /// <summary>
        /// 확장 버튼 색상
        /// </summary>
        private Color expandButtonColor = SystemColors.ControlText;

        /// <summary>
        /// 닫기 버튼 색상
        /// </summary>
        private Color clostButtonColor = SystemColors.ControlText;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 테두리 색상 - BorderColor

        /// <summary>
        /// 테두리 색상
        /// </summary>
        public virtual Color BorderColor
        {
            get
            {
                return this.borderColor;
            }
            set
            {
                if(value.Equals(this.borderColor) == false)
                {
                    this.borderColor = value;

                    OnCustomColorChanged(this, EventArgs.Empty);
                }
            }
        }

        #endregion
        #region 내부 테두리 색상 - InnerBorderColor

        /// <summary>
        /// 내부 테두리 색상
        /// </summary>
        public virtual Color InnerBorderColor
        {
            get
            {
                return this.innerBorderColor;
            }
            set
            {
                if(value.Equals(this.innerBorderColor) == false)
                {
                    this.innerBorderColor = value;

                    OnCustomColorChanged(this, EventArgs.Empty);
                }
            }
        }

        #endregion
        #region 제목 바 시작 색상 - CaptionBarBeginColor

        /// <summary>
        /// 제목 바 시작 색상
        /// </summary>
        public virtual Color CaptionBarBeginColor
        {
            get
            {
                return this.captionBarBeginColor;
            }
            set
            {
                if(value.Equals(this.captionBarBeginColor) == false)
                {
                    this.captionBarBeginColor = value;

                    OnCustomColorChanged(this, EventArgs.Empty);
                }
            }
        }

        #endregion
        #region 제목 바 중간 색상 - CaptionBarMiddleColor

        /// <summary>
        /// 제목 바 중간 색상
        /// </summary>
        public virtual Color CaptionBarMiddleColor
        {
            get
            {
                return this.captionBarMiddleColor;
            }
            set
            {
                if(value.Equals(this.captionBarMiddleColor) == false)
                {
                    this.captionBarMiddleColor = value;

                    OnCustomColorChanged(this, EventArgs.Empty);
                }
            }
        }

        #endregion
        #region 제목 바 종료 색상 - CaptionBarEndColor

        /// <summary>
        /// 제목 바 종료 색상
        /// </summary>
        public virtual Color CaptionBarEndColor
        {
            get
            {
                return this.captionBarEndColor;
            }
            set
            {
                if(value.Equals(this.captionBarEndColor) == false)
                {
                    this.captionBarEndColor = value;

                    OnCustomColorChanged(this, EventArgs.Empty);
                }
            }
        }

        #endregion
        #region 제목 텍스트 - CaptionText

        /// <summary>
        /// 제목 텍스트
        /// </summary>
        public virtual Color CaptionText
        {
            get
            {
                return this.captionText;
            }
            set
            {
                if(value.Equals(this.captionText) == false)
                {
                    this.captionText = value;

                    OnCustomColorChanged(this, EventArgs.Empty);
                }
            }
        }

        #endregion
        #region 확장 버튼 색상 - ExpandButtonColor

        /// <summary>
        /// 확장 버튼 색상
        /// </summary>
        public virtual Color ExpandButtonColor
        {
            get
            {
                return this.expandButtonColor;
            }
            set
            {
                if(value.Equals(this.expandButtonColor) == false)
                {
                    this.expandButtonColor = value;

                    OnCustomColorChanged(this, EventArgs.Empty);
                }
            }
        }

        #endregion
        #region 닫기 버튼 색상 - CloseButtonColor

        /// <summary>
        /// 닫기 버튼 색상
        /// </summary>
        public virtual Color CloseButtonColor
        {
            get
            {
                return this.clostButtonColor;
            }
            set
            {
                if(value.Equals(this.clostButtonColor) == false)
                {
                    this.clostButtonColor = value;

                    OnCustomColorChanged(this, EventArgs.Empty);
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 커스텀 색상 변경시 처리하기 - OnCustomColorChanged(sender, e)

        /// <summary>
        /// 커스텀 색상 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        protected virtual void OnCustomColorChanged(object sender, EventArgs e)
        {
            CustomColorChanged?.Invoke(sender, e);
        }

        #endregion
    }
}