﻿using System.Drawing;
using System.Windows.Forms;
using System.Windows.Forms.Design.Behavior;

namespace TestProject
{
    /// <summary>
    /// 익스팬더 패널 제목 글리프
    /// </summary>
    internal class ExpanderPanelCaptionGlyph : Glyph
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 비헤이비어 서비스
        /// </summary>
        private BehaviorService behaviorService;

        /// <summary>
        /// 익스팬더 패널
        /// </summary>
        private ExpanderPanel expanderPanel;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 경계 사각형 - Bounds

        /// <summary>
        /// 경계 사각형
        /// </summary>
        public override Rectangle Bounds
        {
            get
            {
                Point edgePoint = this.behaviorService.ControlToAdornerWindow(this.expanderPanel);

                Rectangle boundRectangle = new Rectangle
                (
                    edgePoint.X,
                    edgePoint.Y,
                    this.expanderPanel.Width,
                    this.expanderPanel.CaptionHeight
                );

                return boundRectangle;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ExpanderPanelCaptionGlyph(behaviorService, expanderPanel)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="behaviorService">비헤이비어 서비스</param>
        /// <param name="expanderPanel">익스팬더 패널</param>
        public ExpanderPanelCaptionGlyph(BehaviorService behaviorService, ExpanderPanel expanderPanel) : base(new ExpanderPanelCaptionClickBehavior(expanderPanel))
        {
            this.behaviorService = behaviorService;
            this.expanderPanel   = expanderPanel;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 히트 테스트 구하기 - GetHitTest(point)

        /// <summary>
        /// 히트 테스트 구하기
        /// </summary>
        /// <param name="point">포인트</param>
        /// <returns>커서</returns>
        public override Cursor GetHitTest(Point point)
        {
            if((this.expanderPanel != null) && (this.expanderPanel.Expanded == false) && (Bounds.Contains(point)))
            {
                return Cursors.Hand;
            }

            return null;
        }

        #endregion
        #region 페인트하기 - Paint(e)

        /// <summary>
        /// 페인트하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        public override void Paint(PaintEventArgs e)
        {
        }

        #endregion
    }
}