﻿using System;
using System.Collections;
using System.Diagnostics.CodeAnalysis;
using System.Globalization;
using System.Windows.Forms;

using TestProject.Properties;

namespace TestProject
{
    /// <summary>
    /// 익스팬더 패널 컬렉션
    /// </summary>
    [SuppressMessage("Microsoft.Design", "CA1010:CollectionsShouldImplementGenericInterface")]
    public sealed class ExpanderPanelCollection : IList, ICollection, IEnumerable
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 익스팬더 패널 리스트
        /// </summary>
        private ExpanderPanelList expanderPanelList;

        /// <summary>
        /// 컨트롤 컬렉션
        /// </summary>
        private Control.ControlCollection controlCollection;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 동기화 여부 - ICollection.IsSynchronized

        /// <summary>
        /// 동기화 여부
        /// </summary>
        bool ICollection.IsSynchronized
        {
            get
            {
                return ((ICollection)this.controlCollection).IsSynchronized;
            }
        }

        #endregion
        #region 동기 루트 - ICollection.SyncRoot

        /// <summary>
        /// 동기 루트
        /// </summary>
        object ICollection.SyncRoot
        {
            get
            {
                return ((ICollection)this.controlCollection).SyncRoot;
            }
        }

        #endregion
        #region 카운트 - Count

        /// <summary>
        /// 카운트
        /// </summary>
        public int Count
        {
            get
            {
                return this.controlCollection.Count;
            }
        }

        #endregion
        #region 카운트 - ICollection.Count

        /// <summary>
        /// 카운트
        /// </summary>
        int ICollection.Count
        {
            get
            {
                return this.Count;
            }
        }

        #endregion
        #region 읽기 전용 여부 - IsReadOnly

        /// <summary>
        /// 읽기 전용 여부
        /// </summary>
        public bool IsReadOnly
        {
            get
            {
                return this.controlCollection.IsReadOnly;
            }
        }

        #endregion
        #region 읽기 전용 여부 - IList.IsReadOnly

        /// <summary>
        /// 읽기 전용 여부
        /// </summary>
        bool IList.IsReadOnly
        {
            get
            {
                return this.IsReadOnly;
            }
        }

        #endregion
        #region 인덱서 - this[index]

        /// <summary>
        /// 인덱서
        /// </summary>
        /// <param name="index">인덱스</param>
        /// <returns>익스팬더 패널</returns>
        public ExpanderPanel this[int index]
        {
            get
            {
                return (ExpanderPanel)this.controlCollection[index] as ExpanderPanel;
            }
        }

        #endregion
        #region 인덱서 - IList.this[index]

        /// <summary>
        /// 인덱서
        /// </summary>
        /// <param name="index">인덱스</param>
        /// <returns>객체</returns>
        object IList.this[int index]
        {
            get
            {
                return this.controlCollection[index];
            }
            set
            {
            }
        }

        #endregion
        #region 크기 고정 여부 - IList.IsFixedSize

        /// <summary>
        /// 크기 고정 여부
        /// </summary>
        bool IList.IsFixedSize
        {
            get
            {
                return ((IList)this.controlCollection).IsFixedSize;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Internal

        #region 생성자 - ExpanderPanelCollection(expanderPanelList)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="expanderPanelList">익스팬더 패널 리스트</param>
        internal ExpanderPanelCollection(ExpanderPanelList expanderPanelList)
        {
            this.expanderPanelList = expanderPanelList;

            this.controlCollection = this.expanderPanelList.Controls;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 추가하기 - Add(expanderPanel)

        /// <summary>
        /// 추가하기
        /// </summary>
        /// <param name="expanderPanel">익스팬더 패널</param>
        public void Add(ExpanderPanel expanderPanel)
        {
            this.controlCollection.Add(expanderPanel);

            this.expanderPanelList.Invalidate();
        }

        #endregion
        #region 추가하기 - IList.Add(value)

        /// <summary>
        /// 추가하기
        /// </summary>
        /// <param name="value">값</param>
        /// <returns>인덱스</returns>
        int IList.Add(object value)
        {
            ExpanderPanel expanderPanel = value as ExpanderPanel;

            if(expanderPanel == null)
            {
                throw new ArgumentException
                (
                    string.Format
                    (
                        CultureInfo.CurrentUICulture,
                        ConstantHelper.ARGUMENT_EXCAPTION_MESSAGE,
                        typeof(ExpanderPanel).Name
                    )
                );
            }

            this.Add(expanderPanel);

            return this.IndexOf(expanderPanel);
        }

        #endregion
        #region 삽입하기 - Insert(index, expanderPanel)

        /// <summary>
        /// 삽입하기
        /// </summary>
        /// <param name="index">인덱스</param>
        /// <param name="expanderPanel">익스팬더 패널</param>
        public void Insert(int index, ExpanderPanel expanderPanel)
        {
            ((IList)this).Insert(index, (object)expanderPanel);
        }

        #endregion
        #region 삽입하기 - IList.Insert(index, value)

        /// <summary>
        /// 삽입하기
        /// </summary>
        /// <param name="index">인덱스</param>
        /// <param name="value">값</param>
        void IList.Insert(int index, object value)
        {
            if((value is ExpanderPanel) == false)
            {
                throw new ArgumentException
                (
                    string.Format
                    (
                        CultureInfo.CurrentUICulture,
                        ConstantHelper.ARGUMENT_EXCAPTION_MESSAGE,
                        typeof(ExpanderPanel).Name
                    )
                );
            }
        }

        #endregion
        #region 제거하기 - Remove(expanderPanel)

        /// <summary>
        /// 제거하기
        /// </summary>
        /// <param name="expanderPanel">익스팬더 패널</param>
        public void Remove(ExpanderPanel expanderPanel)
        {
            this.controlCollection.Remove(expanderPanel);
        }

        #endregion
        #region 제거하기 - IList.Remove(value)

        /// <summary>
        /// 제거하기
        /// </summary>
        /// <param name="value">값</param>
        void IList.Remove(object value)
        {
            this.Remove(value as ExpanderPanel);
        }

        #endregion
        #region 제거하기 - RemoveAt(index)

        /// <summary>
        /// 제거하기
        /// </summary>
        /// <param name="index">인덱스</param>
        public void RemoveAt(int index)
        {
            this.controlCollection.RemoveAt(index);
        }

        #endregion
        #region 제거하기 - IList.RemoveAt(index)

        /// <summary>
        /// 제거하기
        /// </summary>
        /// <param name="index">인덱스</param>
        void IList.RemoveAt(int index)
        {
            this.RemoveAt(index);
        }

        #endregion
        #region 지우기 - Clear()

        /// <summary>
        /// 지우기
        /// </summary>
        public void Clear()
        {
            this.controlCollection.Clear();
        }

        #endregion
        #region 포함 여부 구하기 - Contains(expanderPanel)

        /// <summary>
        /// 포함 여부 구하기
        /// </summary>
        /// <param name="expanderPanel">익스팬더 패널</param>
        /// <returns>포함 여부</returns>
        public bool Contains(ExpanderPanel expanderPanel)
        {
            return this.controlCollection.Contains(expanderPanel);
        }

        #endregion
        #region 포함 여부 구하기 - IList.Contains(value)

        /// <summary>
        /// 포함 여부 구하기
        /// </summary>
        /// <param name="value">값</param>
        /// <returns>포함 여부</returns>
        bool IList.Contains(object value)
        {
            return this.Contains(value as ExpanderPanel);
        }

        #endregion
        #region 인덱스 구하기 - IndexOf(expanderPanel)

        /// <summary>
        /// 인덱스 구하기
        /// </summary>
        /// <param name="expanderPanel">익스팬더 패널</param>
        /// <returns>인덱스</returns>
        public int IndexOf(ExpanderPanel expanderPanel)
        {
            return this.controlCollection.IndexOf(expanderPanel);
        }

        #endregion
        #region 인덱스 구하기 - IList.IndexOf(value)

        /// <summary>
        /// 인덱스 구하기
        /// </summary>
        /// <param name="value">값</param>
        /// <returns>인덱스</returns>
        int IList.IndexOf(object value)
        {
            return this.IndexOf(value as ExpanderPanel);
        }

        #endregion
        #region 열거자 구하기 - GetEnumerator()

        /// <summary>
        /// 열거자 구하기
        /// </summary>
        /// <returns>열거자</returns>
        public IEnumerator GetEnumerator()
        {
            return this.controlCollection.GetEnumerator();
        }

        #endregion
        #region 복사하기 - CopyTo(expanderPanelArray, index)

        /// <summary>
        /// 복사하기
        /// </summary>
        /// <param name="expanderPanelArray">익스팬더 패널 배열</param>
        /// <param name="index">인덱스</param>
        public void CopyTo(ExpanderPanel[] expanderPanelArray, int index)
        {
            this.controlCollection.CopyTo(expanderPanelArray, index);
        }

        #endregion
        #region 복사하기 - ICollection.CopyTo(array, index)

        /// <summary>
        /// 복사하기
        /// </summary>
        /// <param name="array">배열</param>
        /// <param name="index">인덱스</param>
        void ICollection.CopyTo(Array array, int index)
        {
            ((ICollection)this.controlCollection).CopyTo(array, index);
        }

        #endregion
    }
}