﻿using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Windows.Forms.Design.Behavior;

namespace TestProject
{
    /// <summary>
    /// 익스팬더 패널 디자이너
    /// </summary>
    internal class ExpanderPanelDesigner : ScrollableControlDesigner
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 테두리 펜
        /// </summary>
        private Pen borderPen = new Pen(Color.FromKnownColor(KnownColor.ControlDarkDark));

        /// <summary>
        /// 장식자
        /// </summary>
        private Adorner adorner;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ExpanderPanelDesigner()

        /// <summary>
        /// 생성자
        /// </summary>
        public ExpanderPanelDesigner()
        {
            this.borderPen.DashStyle = DashStyle.Dash;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 초기화하기 - Initialize(component)

        /// <summary>
        /// 초기화하기
        /// </summary>
        /// <param name="component">컴포넌트</param>
        public override void Initialize(IComponent component)
        {
            base.Initialize(component);

            ExpanderPanel expanderPanel = Control as ExpanderPanel;

            if(expanderPanel != null)
            {
                this.adorner = new Adorner();

                BehaviorService.Adorners.Add(this.adorner);

                this.adorner.Glyphs.Add(new ExpanderPanelCaptionGlyph(BehaviorService, expanderPanel));
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 리소스 해제하기 - Dispose(disposing)

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        /// <param name="disposing">리소스 해제 여부</param>
        protected override void Dispose(bool disposing)
        {
            try
            {
                if(disposing)
                {
                    if(this.borderPen != null)
                    {
                        this.borderPen.Dispose();
                    }

                    if(this.adorner != null)
                    {
                        if(BehaviorService != null)
                        {
                            BehaviorService.Adorners.Remove(this.adorner);
                        }
                    }
                }
            }
            finally
            {
                base.Dispose(disposing);
            }
        }

        #endregion

        #region 장식 페인트시 처리하기 - OnPaintAdornments(e)

        /// <summary>
        /// 장식 페인트시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnPaintAdornments(PaintEventArgs e)
        {
            base.OnPaintAdornments(e);

            e.Graphics.DrawRectangle
            (
                this.borderPen,
                0,
                0,
                this.Control.Width  - 2,
                this.Control.Height - 2
            );
        }

        #endregion

        #region 속성 필터 처리하기 - PostFilterProperties(propertyDictionary)

        /// <summary>
        /// 속성 필터 처리하기
        /// </summary>
        /// <param name="propertyDictionary">속성 딕셔너리</param>
        protected override void PostFilterProperties(IDictionary propertyDictionary)
        {
            base.PostFilterProperties(propertyDictionary);

            propertyDictionary.Remove("AccessibilityObject");
            propertyDictionary.Remove("AccessibleDefaultActionDescription");
            propertyDictionary.Remove("AccessibleDescription");
            propertyDictionary.Remove("AccessibleName");
            propertyDictionary.Remove("AccessibleRole");
            propertyDictionary.Remove("AllowDrop");
            propertyDictionary.Remove("Anchor");
            propertyDictionary.Remove("AntiAliasing");
            propertyDictionary.Remove("AutoScroll");
            propertyDictionary.Remove("AutoScrollMargin");
            propertyDictionary.Remove("AutoScrollMinSize");
            propertyDictionary.Remove("BackColor");
            propertyDictionary.Remove("BackgroundImage");
            propertyDictionary.Remove("BackgroundImageLayout");
            propertyDictionary.Remove("CausesValidation");
            propertyDictionary.Remove("ContextMenuStrip");
            propertyDictionary.Remove("Dock");
            propertyDictionary.Remove("GenerateMember");
            propertyDictionary.Remove("ImeMode");
            propertyDictionary.Remove("Location");
            propertyDictionary.Remove("MaximumSize");
            propertyDictionary.Remove("MinimumSize");
        }

        #endregion
    }
}