﻿using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using System.Windows.Forms.Design;

namespace TestProject
{
    /// <summary>
    /// 익스팬더 패널 리스트 디자이너
    /// </summary>
    internal class ExpanderPanelListDesigner : ParentControlDesigner
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 테두리 펜
        /// </summary>
        private Pen borderPen = new Pen(Color.FromKnownColor(KnownColor.ControlDarkDark));

        /// <summary>
        /// 익스팬더 패널 리스트
        /// </summary>
        private ExpanderPanelList expanderPanelList;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 액션 리스트 - ActionLists

        /// <summary>
        /// 액션 리스트
        /// </summary>
        public override DesignerActionListCollection ActionLists
        {
            get
            {
                DesignerActionListCollection collection = new DesignerActionListCollection();

                collection.Add(new ExpanderPanelListDesignerActionList(Component));

                return collection;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ExpanderPanelListDesigner()

        /// <summary>
        /// 생성자
        /// </summary>
        public ExpanderPanelListDesigner()
        {
            this.borderPen.DashStyle = DashStyle.Dash;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 초기화하기 - Initialize(component)

        /// <summary>
        /// 초기화하기
        /// </summary>
        /// <param name="component">컴포넌트</param>
        public override void Initialize(IComponent component)
        {
            base.Initialize(component);

            this.expanderPanelList = (ExpanderPanelList)Control;

            this.expanderPanelList.AutoScroll = false;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 리소스 해제하기 - Dispose(disposing)

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        /// <param name="disposing">리소스 해제 여부</param>
        protected override void Dispose(bool disposing)
        {
            try
            {
                if(disposing)
                {
                    if(this.borderPen != null)
                    {
                        this.borderPen.Dispose();
                    }
                }
            }
            finally
            {
                base.Dispose(disposing);
            }
        }

        #endregion

        #region 장식 페인트시 처리하기 - OnPaintAdornments(e)

        /// <summary>
        /// 장식 페인트시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnPaintAdornments(PaintEventArgs e)
        {
            base.OnPaintAdornments(e);

            e.Graphics.DrawRectangle(this.borderPen, 0, 0, this.expanderPanelList.Width - 2, this.expanderPanelList.Height - 2);
        }

        #endregion
    }
}