﻿using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing.Design;
using System.Reflection;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 익스팬더 패널 리스트 디자이너 액션 리스트
    /// </summary>
    public class ExpanderPanelListDesignerActionList : DesignerActionList
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 패널 스타일 - PanelStyle

        /// <summary>
        /// 패널 스타일
        /// </summary>
        public PanelStyle PanelStyle
        {
            get
            {
                return ExpanderPanelList.PanelStyle;
            }
            set
            {
                SetProperty("PanelStyle", value);
            }
        }

        #endregion
        #region 색상 스킴 - ColorScheme

        /// <summary>
        /// 색상 스킴
        /// </summary>
        public ColorScheme ColorScheme
        {
            get
            {
                return ExpanderPanelList.ColorScheme;
            }
            set
            {
                SetProperty("ColorScheme", value);
            }
        }

        #endregion
        #region 테두리 표시 여부 - ShowBorder

        /// <summary>
        /// 테두리 표시 여부
        /// </summary>
        public bool ShowBorder
        {
            get
            {
                return ExpanderPanelList.ShowBorder;
            }
            set
            {
                SetProperty("ShowBorder", value);
            }
        }

        #endregion
        #region 제목 스타일 - CaptionStyle

        /// <summary>
        /// 제목 스타일
        /// </summary>
        public CaptionStyle CaptionStyle
        {
            get
            {
                return ExpanderPanelList.CaptionStyle;
            }
            set
            {
                SetProperty("CaptionStyle", value);
            }
        }

        #endregion
        #region 확장 버튼 표시 여부 - ShowExpandButton

        /// <summary>
        /// 확장 버튼 표시 여부
        /// </summary>
        public bool ShowExpandButton
        {
            get
            {
                return ExpanderPanelList.ShowExpandButton;
            }
            set
            {
                SetProperty("ShowExpandButton", value);
            }
        }

        #endregion
        #region 닫기 버튼 표시 여부 - ShowCloseButton

        /// <summary>
        /// 닫기 버튼 표시 여부
        /// </summary>
        public bool ShowCloseButton
        {
            get
            {
                return ExpanderPanelList.ShowCloseButton;
            }
            set
            {
                SetProperty("ShowCloseButton", value);
            }
        }

        #endregion
        #region 익스팬더 패널 컬렉션 - ExpanderPanelCollection

        /// <summary>
        /// 익스팬더 패널 컬렉션
        /// </summary>
        [Editor(typeof(ExpanderPanelCollectionEditor), typeof(UITypeEditor))]
        public ExpanderPanelCollection ExpanderPanelCollection
        {
            get
            {
                return ExpanderPanelList.ExpanderPanelCollection;
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 익스팬더 패널 리스트 - ExpanderPanelList

        /// <summary>
        /// 익스팬더 패널 리스트
        /// </summary>
        private ExpanderPanelList ExpanderPanelList
        {
            get
            {
                return (ExpanderPanelList)Component;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ExpanderPanelListDesignerActionList(component)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="component">컴포넌트</param>
        public ExpanderPanelListDesignerActionList(IComponent component) : base(component)
        {
            base.AutoShow = true;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 카테고리 구하기 - GetCategory(source, propertyName)

        /// <summary>
        /// 카테고리 구하기
        /// </summary>
        /// <param name="source">소스</param>
        /// <param name="propertyName">속성명</param>
        /// <returns>카테고리</returns>
        private static string GetCategory(object source, string propertyName)
        {
            PropertyInfo propertyInfo = source.GetType().GetProperty(propertyName);

            CategoryAttribute attribute = (CategoryAttribute)propertyInfo.GetCustomAttributes(typeof(CategoryAttribute), false)[0];

            if(attribute == null)
            {
                return null;
            }
            else
            {
                return attribute.Category;
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 도킹 스타일 토글하기 - ToggleDockStyle()

        /// <summary>
        /// 도킹 스타일 토글하기
        /// </summary>
        public void ToggleDockStyle()
        {
            if(ExpanderPanelList.Dock != DockStyle.Fill)
            {
                SetProperty("Dock", DockStyle.Fill);
            }
            else
            {
                SetProperty("Dock", DockStyle.None);
            }
        }

        #endregion

        #region 정렬 액션 항목 목록 구하기 - GetSortedActionItems()

        /// <summary>
        /// 정렬 액션 항목 목록 구하기
        /// </summary>
        /// <returns></returns>
        public override DesignerActionItemCollection GetSortedActionItems()
        {
            DesignerActionItemCollection collection = new DesignerActionItemCollection();

            collection.Add
            (
                new DesignerActionMethodItem
                (
                    this,
                    "ToggleDockStyle",
                    GetDockStyleText(),
                    "Design",
                    "Dock or undock this control in it's parent container.",
                    true
                )
            );
            
            collection.Add
            (
                new DesignerActionPropertyItem
                (
                    "ShowBorder",
                    "Show Border",
                    GetCategory(ExpanderPanelList, "ShowBorder")
                )
            );

            collection.Add
            (
                new DesignerActionPropertyItem
                (
                    "ShowExpandButton",
                    "Show ExpandButton",
                    GetCategory(ExpanderPanelList, "ShowExpandButton")
                )
            );

            collection.Add
            (
                new DesignerActionPropertyItem
                (
                    "ShowCloseButton",
                    "Show CloseButton",
                    GetCategory(ExpanderPanelList, "ShowCloseButton")
                )
            );
            
            collection.Add
            (
                new DesignerActionPropertyItem
                (
                    "PanelStyle",
                    "Select PanelStyle",
                    GetCategory(ExpanderPanelList, "PanelStyle")
                )
            );

            collection.Add
            (
                new DesignerActionPropertyItem
                (
                    "ColorScheme",
                    "Select ColorScheme",
                    GetCategory(ExpanderPanelList, "ColorScheme")
                )
            );

            collection.Add
            (
                new DesignerActionPropertyItem
                (
                    "CaptionStyle",
                    "Select CaptionStyle",
                    GetCategory(ExpanderPanelList, "CaptionStyle")
                )
            );

            collection.Add
            (
                new DesignerActionPropertyItem
                (
                    "ExpanderPanelCollection",
                    "Edit ExpanderPanelCollection",
                    GetCategory(ExpanderPanelList, "ExpanderPanelCollection")
                )
            );
        
            return collection;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 도킹 스타일 텍스트 구하기 - GetDockStyleText()

        /// <summary>
        /// 도킹 스타일 텍스트 구하기
        /// </summary>
        /// <returns>도킹 스타일 텍스트</returns>
        private string GetDockStyleText()
        {
            if(ExpanderPanelList.Dock == DockStyle.Fill)
            {
                return "Undock in parent container";
            }
            else
            {
                return "Dock in parent container";
            }
        }

        #endregion

        #region 속성 설정하기 - SetProperty(propertyName, value)

        /// <summary>
        /// 속성 설정하기
        /// </summary>
        /// <param name="propertyName">속성명</param>
        /// <param name="value">값</param>
        private void SetProperty(string propertyName, object value)
        {
            PropertyDescriptor propertyDescriptor = TypeDescriptor.GetProperties(ExpanderPanelList)[propertyName];

            propertyDescriptor.SetValue(ExpanderPanelList, value);
        }

        #endregion
    }
}