﻿using System.ComponentModel;
using System.ComponentModel.Design;
using System.Reflection;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 패널 디자이너 액션 리스트
    /// </summary>
    public class PanelDesignerActionList : DesignerActionList
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 패널 스타일 - PanelStyle

        /// <summary>
        /// 패널 스타일
        /// </summary>
        public PanelStyle PanelStyle
        {
            get
            {
                return Panel.PanelStyle;
            }
            set
            {
                SetProperty("PanelStyle", value);
            }
        }

        #endregion
        #region 익스팬더 패널 프로페셔널 스타일 - ShowExpanderPanelProfessionalStyle

        /// <summary>
        /// 익스팬더 패널 프로페셔널 스타일
        /// </summary>
        public bool ShowExpanderPanelProfessionalStyle
        {
            get
            {
                return Panel.ShowExpanderPanelProfessionalStyle;
            }
            set
            {
                SetProperty("ShowExpanderPanelProfessionalStyle", value);
            }
        }

        #endregion
        #region 색상 스킴 - ColorScheme

        /// <summary>
        /// 색상 스킴
        /// </summary>
        public ColorScheme ColorScheme
        {
            get
            {
                return this.Panel.ColorScheme;
            }
            set
            {
                SetProperty("ColorScheme", value);
            }
        }

        #endregion
        #region 투명 배경 표시 여부 - ShowTransparentBackground

        /// <summary>
        /// 투명 배경 표시 여부
        /// </summary>
        public bool ShowTransparentBackground
        {
            get
            {
                return Panel.ShowTransparentBackground;
            }
            set
            {
                SetProperty("ShowTransparentBackground", value);
            }
        }

        #endregion
        #region 제목 바 표시 여부 - ShowCaptionBar

        /// <summary>
        /// 제목 바 표시 여부
        /// </summary>
        public bool ShowCaptionBar
        {
            get
            {
                return Panel.ShowCaptionBar;
            }
            set
            {
                SetProperty("ShowCaptionBar", value);
            }
        }

        #endregion
        #region 확장 버튼 표시 여부 - ShowExpandButton

        /// <summary>
        /// 확장 버튼 표시 여부
        /// </summary>
        public bool ShowExpandButton
        {
            get
            {
                return Panel.ShowExpandButton;
            }
            set
            {
                SetProperty("ShowExpandButton", value);
            }
        }

        #endregion
        #region 닫기 버튼 표시 여부 - ShowCloseButton

        /// <summary>
        /// 닫기 버튼 표시 여부
        /// </summary>
        public bool ShowCloseButton
        {
            get
            {
                return Panel.ShowCloseButton;
            }
            set
            {
                SetProperty("ShowCloseButton", value);
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 패널 - Panel

        /// <summary>
        /// 패널
        /// </summary>
        private Panel Panel
        {
            get
            {
                return (Panel)Component;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - PanelDesignerActionList(component)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="component">컴포넌트</param>
        public PanelDesignerActionList(IComponent component) : base(component)
        {
            base.AutoShow = true;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 카테고리 구하기 - GetCategory(source, propertyName)

        /// <summary>
        /// 카테고리 구하기
        /// </summary>
        /// <param name="source">소스 객체</param>
        /// <param name="propertyName">속성명</param>
        /// <returns>카테고리</returns>
        private static string GetCategory(object source, string propertyName)
        {
            PropertyInfo property = source.GetType().GetProperty(propertyName);

            CategoryAttribute attribute = (CategoryAttribute)property.GetCustomAttributes(typeof(CategoryAttribute), false)[0];

            if(attribute == null)
            {
                return null;
            }

            return attribute.Category;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 정렬 액션 목록 구하기 - GetSortedActionItems()

        /// <summary>
        /// 정렬 액션 목록 구하기
        /// </summary>
        /// <returns>정렬 액션 목록 </returns>
        public override DesignerActionItemCollection GetSortedActionItems()
        {
            DesignerActionItemCollection collection = new DesignerActionItemCollection();

            collection.Add
            (
                new DesignerActionMethodItem
                (
                    this,
                    "ToggleDockStyle",
                    GetDockStyleText(),
                    "Design",
                    "Dock or undock this control in it's parent container.",
                    true
                )
            );

            collection.Add
            (
                new DesignerActionPropertyItem
                (
                    "ShowTransparentBackground",
                    "Show transparent backcolor",
                    GetCategory(Panel, "ShowTransparentBackground")
                )
            );

            collection.Add
            (
                new DesignerActionPropertyItem
                (
                    "ShowExpanderPanelProfessionalStyle",
                    "Show the XPanderPanels professional colorscheme",
                    GetCategory(Panel, "ShowExpanderPanelProfessionalStyle")
                )
            );

            collection.Add
            (
                new DesignerActionPropertyItem
                (
                    "ShowCaptionBar",
                    "Show the captionbar on top of the panel",
                    GetCategory(Panel, "ShowCaptionBar")
                )
            );

            collection.Add
            (
                new DesignerActionPropertyItem
                (
                    "ShowExpandButton",
                    "Show the expand panel icon (not at DockStyle.None or DockStyle.Fill)",
                    GetCategory(Panel, "ShowExpandButton")
                )
            );

            collection.Add
            (
                new DesignerActionPropertyItem
                (
                    "ShowCloseButton",
                    "Show the close panel icon (not at DockStyle.None or DockStyle.Fill)",
                    GetCategory(Panel, "ShowCloseButton")
                )
            );

            collection.Add
            (
                new DesignerActionPropertyItem
                (
                    "PanelStyle",
                    "Select PanelStyle",
                    GetCategory(Panel, "PanelStyle")
                )
            );

            collection.Add
            (
                new DesignerActionPropertyItem
                (
                    "ColorScheme",
                    "Select ColorScheme",
                    GetCategory(Panel, "ColorScheme")
                )
            );

            return collection;
        }

        #endregion
        #region 도킹 스타일 토글하기 - ToggleDockStyle()

        /// <summary>
        /// 도킹 스타일 토글하기
        /// </summary>
        public void ToggleDockStyle()
        {
            if(Panel.Dock != DockStyle.Fill)
            {
                SetProperty("Dock", DockStyle.Fill);
            }
            else
            {
                SetProperty("Dock", DockStyle.None);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 도킹 스타일 텍스트 구하기 - GetDockStyleText()

        /// <summary>
        /// 도킹 스타일 텍스트 구하기
        /// </summary>
        /// <returns>도킹 스타일 텍스트</returns>
        private string GetDockStyleText()
        {
            if(Panel.Dock == DockStyle.Fill)
            {
                return "Undock in parent container";
            }
            else
            {
                return "Dock in parent container";
            }
        }

        #endregion
        #region 속성 설정하기 - SetProperty(propertyName, value)

        /// <summary>
        /// 속성 설정하기
        /// </summary>
        /// <param name="propertyName">속성명</param>
        /// <param name="value">값</param>
        private void SetProperty(string propertyName, object value)
        {
            PropertyDescriptor propertyDescriptor = TypeDescriptor.GetProperties(Panel)[propertyName];

            propertyDescriptor.SetValue(Panel, value);
        }

        #endregion
    }
}