﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Globalization;
using System.Windows.Forms;

using TestProject.Properties;

namespace TestProject
{
    /// <summary>
    /// 진행 바
    /// </summary>
    [ToolboxBitmap(typeof(System.Windows.Forms.ProgressBar))]
    public partial class ProgressBar : Control
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 테두리 색상 변경시 이벤트 - BorderColorChanged

        /// <summary>
        /// 테두리 색상 변경시 이벤트
        /// </summary>
        public event EventHandler<EventArgs> BorderColorChanged;

        #endregion
        #region 배경 색상 변경시 이벤트 핸들러 - BackgroundColorChanged

        /// <summary>
        /// 배경 색상 변경시 이벤트 핸들러
        /// </summary>
        public event EventHandler<EventArgs> BackgroundColorChanged;

        #endregion
        #region 값 색상 변경시 이벤트 - ValueColorChanged

        /// <summary>
        /// 값 색상 변경시 이벤트
        /// </summary>
        public event EventHandler<EventArgs> ValueColorChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 테두리 색상
        /// </summary>
        private Color borderColor;

        /// <summary>
        /// 배경 색상
        /// </summary>
        private Color backgroundColor;

        /// <summary>
        /// 값 색상
        /// </summary>
        private Color valueColor;

        /// <summary>
        /// 최소 값
        /// </summary>
        private int minimumValue;

        /// <summary>
        /// 최대 값
        /// </summary>
        private int maximumValue;

        /// <summary>
        /// 값
        /// </summary>
        private int value;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 테두리 색상 - BorderColor

        /// <summary>
        /// 테두리 색상
        /// </summary>
        [Category("ProgressBar")]
        [Browsable(true)]
        public Color BorderColor
        {
            get
            {
                return this.borderColor;
             }
            set
            {
                if(this.borderColor != value)
                {
                    this.borderColor = value;

                    OnBorderColorChanged(this, EventArgs.Empty);
                }
            }
        }

        #endregion
        #region 배경 색상 - BackgroundColor

        /// <summary>
        /// 배경 색상
        /// </summary>
        [Category("ProgressBar")]
        [Browsable(true)]
        public Color BackgroundColor
        {
            get
            {
                return this.backgroundColor;
            }
            set
            {
                if(this.backgroundColor != value)
                {
                    this.backgroundColor = value;

                    OnBackgroundColorChanged(this, EventArgs.Empty);
                }
            }
        }

        #endregion
        #region 배경 색상 - BackColor

        /// <summary>
        /// 배경 색상
        /// </summary>
        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public new Color BackColor
        {
            get
            {
                return base.BackColor;
            }
            set
            {
                base.BackColor = value;
            }
        }

        #endregion
        #region 값 색상 - ValueColor

        /// <summary>
        /// 값 색상
        /// </summary>
        [Category("ProgressBar")]
        [Browsable(true)]
        public Color ValueColor
        {
            get
            {
                return this.valueColor;
            }
            set
            {
                if(this.valueColor != value)
                {
                    this.valueColor = value;

                    OnValueColorChanged(this, EventArgs.Empty);
                }
            }
        }

        #endregion

        #region 최소 값 - MinimumValue

        /// <summary>
        /// 최소 값
        /// </summary>
        [Category("ProgressBar")]
        [Browsable(true)]
        public int MinimumValue
        {
            get
            {
                return this.minimumValue;
            }
            set
            {
                if(this.minimumValue != value)
                {
                    if(value < 0)
                    {
                        object[] argumentArray = new object[]
                        {
                            "MinimumValue",
                            value.ToString(CultureInfo.CurrentCulture),
                            "MinimumValue"
                        };

                        throw new ArgumentOutOfRangeException
                        (
                            "MinimumValue",
                            string.Format
                            (
                                CultureInfo.InvariantCulture,
                                ConstantHelper.INVALID_LOW_BOUND_ARGUMENT,
                                argumentArray
                            )
                        );
                    }

                    if(this.maximumValue < value)
                    {
                        this.maximumValue = value;
                    }

                    this.minimumValue = value;

                    if(this.value < this.minimumValue)
                    {
                        this.value = this.minimumValue;
                    }

                    UpdatePosition();
                }
            }
        }

        #endregion
        #region 최대 값 - MaximumValue

        /// <summary>
        /// 최대 값
        /// </summary>
        [Category("ProgressBar")]
        [Browsable(true)]
        public int MaximumValue
        {
            get
            {
                return this.maximumValue;
            }
            set
            {
                if(this.maximumValue != value)
                {
                    if(value < 0)
                    {
                        object[] argumentArray = new object[]
                        {
                            "MaximumValue",
                            value.ToString(CultureInfo.CurrentCulture),
                            "MaximumValue"
                        };

                        throw new ArgumentOutOfRangeException
                        (
                            "MaximumValue",
                            string.Format
                            (
                                CultureInfo.InvariantCulture,
                                ConstantHelper.INVALID_LOW_BOUND_ARGUMENT,
                                argumentArray
                            )
                        );
                    }

                    if(this.minimumValue > value)
                    {
                        this.minimumValue = value;
                    }

                    this.maximumValue = value;

                    if(this.value > this.maximumValue)
                    {
                        this.value = this.maximumValue;
                    }

                    UpdatePosition();
                }
            }
        }

        #endregion
        #region 값 - Value

        /// <summary>
        /// 값
        /// </summary>
        [Category("Value")]
        [Browsable(true)]
        public int Value
        {
            get
            {
                return this.value;
            }
            set
            {
                if(this.value != value)
                {
                    if((value < this.minimumValue) || (value > this.maximumValue))
                    {
                        throw new ArgumentOutOfRangeException
                        (
                            "Value",
                            string.Format
                            (
                                CultureInfo.InvariantCulture,
                                ConstantHelper.INVALID_BOUND_ARGUMENT,
                                new object[]
                                {
                                    "Value",
                                    value.ToString(CultureInfo.CurrentCulture),
                                    "'minimum value'",
                                    "'maximum value'"
                                }
                            )
                        );
                    }

                    this.value = value;

                    UpdatePosition();
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ProgressBar()

        /// <summary>
        /// 생성자
        /// </summary>
        public ProgressBar()
        {
            this.SetStyle(ControlStyles.AllPaintingInWmPaint        , true);
            this.SetStyle(ControlStyles.UserPaint                   , true);
            this.SetStyle(ControlStyles.OptimizedDoubleBuffer       , true);
            this.SetStyle(ControlStyles.ResizeRedraw                , true);
            this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);

            InitializeComponent();

            this.maximumValue = 100;

            this.backgroundColor = Color.FromArgb(20, 20, 255);

            this.valueColor = Color.FromArgb(255, 0, 255);

            this.borderColor = SystemColors.ActiveBorder;

            this.BackColor = Color.Transparent;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 페인트시 처리하기 - OnPaint(e)

        /// <summary>
        /// 페인트시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnPaint(PaintEventArgs e)
        {
            using(AntiAliasHelper antiAliasHelper = new AntiAliasHelper(e.Graphics))
            {
                Graphics graphics = e.Graphics;

                DrawProgressBar
                (
                    graphics,
                    ClientRectangle,
                    this.backgroundColor,
                    this.valueColor,
                    this.borderColor,
                    RightToLeft,
                    MinimumValue,
                    MaximumValue,
                    Value
                );

                if(string.IsNullOrEmpty(Text) == false)
                {
                    using(ClearTypeGridFitHelper clearTypeGridFitHelper = new ClearTypeGridFitHelper(graphics))
                    {
                        using(SolidBrush textBrush = new SolidBrush(this.ForeColor))
                        {
                            using(StringFormat stringFormat = new StringFormat())
                            {
                                stringFormat.FormatFlags = StringFormatFlags.NoWrap;

                                if(RightToLeft == RightToLeft.Yes)
                                {
                                    stringFormat.FormatFlags |= StringFormatFlags.DirectionRightToLeft;
                                }

                                stringFormat.Trimming      = StringTrimming.EllipsisCharacter;
                                stringFormat.LineAlignment = StringAlignment.Center;
                                stringFormat.Alignment     = StringAlignment.Center;

                                Rectangle stringRectangle = ClientRectangle;

                                graphics.DrawString(Text, Font, textBrush, stringRectangle, stringFormat);
                            }
                        }
                    }
                }
            }
        }

        #endregion

        #region 테두리 색상 변경시 처리하기 - OnBorderColorChanged(sender, e)

        /// <summary>
        /// 테두리 색상 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
		protected virtual void OnBorderColorChanged(object sender, EventArgs e)
        {
            Invalidate(true);

            BorderColorChanged?.Invoke(sender, e);
        }

        #endregion
        #region 배경 색상 변경시 처리하기 - OnBackgroundColorChanged(sender, e)

        /// <summary>
        /// 배경 색상 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        protected virtual void OnBackgroundColorChanged(object sender, EventArgs e)
        {
            Invalidate();

            BackgroundColorChanged?.Invoke(sender, e);
        }

        #endregion
        #region 값 색상 변경시 처리하기 - OnValueColorChanged(sender, e)

        /// <summary>
        /// 값 색상 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        protected virtual void OnValueColorChanged(object sender, EventArgs e)
        {
            Invalidate(true);

            ValueColorChanged?.Invoke(sender, e);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 위치 업데이트하기 - UpdatePosition()

        /// <summary>
        /// 위치 업데이트하기
        /// </summary>
        private void UpdatePosition()
        {
            Invalidate(true);
        }

        #endregion

        #region 배경 사각형 구하기 - GetBackgroundRectangle(clientRectangle)

        /// <summary>
        /// 배경 사각형 구하기
        /// </summary>
        /// <param name="clientRectangle">클라이언트 사각형</param>
        /// <returns>배경 사각형</returns>
        private static Rectangle GetBackgroundRectangle(Rectangle clientRectangle)
        {
            Rectangle backgroundRectangle = clientRectangle;

            backgroundRectangle.Inflate(-1, -1);

            return backgroundRectangle;
        }

        #endregion
        #region 값 사각형 구하기 - GetValueRectangle(backgroundRectangle, rightToLeft, minimumValue, maximumValue, value)

        /// <summary>
        /// 값 사각형 구하기
        /// </summary>
        /// <param name="backgroundRectangle">배경 사각형</param>
        /// <param name="rightToLeft">오른쪽→왼쪽 출력 여부</param>
        /// <param name="minimumValue">최소 값</param>
        /// <param name="maximumValue">최대 값</param>
        /// <param name="value">값</param>
        /// <returns>값 사각형</returns>
        private static Rectangle GetValueRectangle(Rectangle backgroundRectangle, RightToLeft rightToLeft, int minimumValue, int maximumValue, int value)
        {
            Rectangle valueRectangle = backgroundRectangle;

            int progressRange = maximumValue - minimumValue;

            int valueRange = value - minimumValue;

            int range = (int)((float)valueRange / (float)progressRange * backgroundRectangle.Width);

            valueRectangle.Width = range;

            if(rightToLeft == RightToLeft.Yes)
            {
                valueRectangle.X = backgroundRectangle.Width - valueRectangle.Width;
            }

            return valueRectangle;
        }

        #endregion

        #region 배경 경로 구하기 - GetBackgroundPath(boundRectangle, radius)

        /// <summary>
        /// 배경 경로 구하기
        /// </summary>
        /// <param name="boundRectangle">경계 사각형</param>
        /// <param name="radius">반지름</param>
        /// <returns>배경 경로</returns>
        private static GraphicsPath GetBackgroundPath(Rectangle boundRectangle, int radius)
        {
            int x      = boundRectangle.X;
            int y      = boundRectangle.Y;
            int width  = boundRectangle.Width;
            int height = boundRectangle.Height;

            GraphicsPath graphicsPath = new GraphicsPath();

            graphicsPath.AddArc(x                 , y                  , radius, radius, 180, 90);
            graphicsPath.AddArc(x + width - radius, y                  , radius, radius, 270, 90);
            graphicsPath.AddArc(x + width - radius, y + height - radius, radius, radius, 0  , 90);
            graphicsPath.AddArc(x                 , y + height - radius, radius, radius, 90 , 90);

            graphicsPath.CloseFigure();

            return graphicsPath;
        }

        #endregion
        #region 값 경로 구하기 - GetValuePath(boundRectangle, rightToLeft, radius)

        /// <summary>
        /// 값 경로 구하기
        /// </summary>
        /// <param name="boundRectangle">경계 사각형</param>
        /// <param name="rightToLeft">오른쪽→왼쪽 출력 여부</param>
        /// <param name="radius">반 지름</param>
        /// <returns>값 경로</returns>
        private static GraphicsPath GetValuePath(Rectangle boundRectangle, RightToLeft rightToLeft, int radius)
        {
            int x      = boundRectangle.X;
            int y      = boundRectangle.Y;
            int width  = boundRectangle.Width;
            int height = boundRectangle.Height;

            GraphicsPath graphicsPath = new GraphicsPath();

            if(rightToLeft == RightToLeft.No)
            {
                graphicsPath.AddArc(x, y, radius, radius, 180, 90);

                graphicsPath.AddLine(x + radius, y, x + width, y);

                graphicsPath.AddLine(x + width, y, x + width, y + height);

                graphicsPath.AddArc(x, y + height - radius, radius, radius, 90, 90);
            }
            else
            {
                graphicsPath.AddLine(x, y, width - radius, y);

                graphicsPath.AddArc(x + width - radius, y, radius, radius, 270, 90);

                graphicsPath.AddLine(x + width, y + radius, x + width, y + radius + height - (2 * radius));

                graphicsPath.AddArc(x + width - radius, y + radius + height - (2 * radius), radius, radius, 360, 90);

                graphicsPath.AddLine(x + width - radius, y + height, x, y + height);
            }

            graphicsPath.CloseFigure();

            return graphicsPath;
        }

        #endregion

        #region 배경 브러시 구하기 - GetBackgroundBrush(boundRectangle, color)

        /// <summary>
        /// 배경 브러시 구하기
        /// </summary>
        /// <param name="boundRectangle">경계 사각형</param>
        /// <param name="color">색상</param>
        /// <returns>배경 브러시</returns>
        private static LinearGradientBrush GetBackgroundBrush(Rectangle boundRectangle, Color color)
        {
            if(IsZeroWidthOrHeight(boundRectangle))
            {
                return null;
            }

            LinearGradientBrush brush = brush = new LinearGradientBrush(boundRectangle, Color.White, color, LinearGradientMode.Vertical);

            if(brush != null)
            {
                Blend blend = new Blend();

                blend.Positions = new float[] { 0.0f, 0.2f, 0.3f, 0.5f, 0.6f, 0.8f, 1.0f };
                blend.Factors   = new float[] { 0.3f, 0.4f, 0.5f, 0.8f, 1.0f, 1.0f, 0.9f };

                brush.Blend = blend;
            }

            return brush;
        }

        #endregion

        #region 너비 또는 높이 길이 0 여부 구하기 - IsZeroWidthOrHeight(rectangle)

        /// <summary>
        /// 너비 또는 높이 길이 0 여부 구하기
        /// </summary>
        /// <param name="rectangle">사각형</param>
        /// <returns>너비 또는 높이 길이 0 여부</returns>
        private static bool IsZeroWidthOrHeight(Rectangle rectangle)
        {
            if(rectangle.Width != 0)
            {
                return (rectangle.Height == 0);
            }

            return true;
        }

        #endregion

        #region 진행 바 그리기 - DrawProgressBar(graphics, clientRectangle, backgroundColor, valueColor, borderColor, rightToLeft, minimumValue, maximumValue, value)

        /// <summary>
        /// 진행 바 그리기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="clientRectangle">클라이언트 사각형</param>
        /// <param name="backgroundColor">배경 색상</param>
        /// <param name="valueColor">값 색상</param>
        /// <param name="borderColor">테두리 색상</param>
        /// <param name="rightToLeft">오른쪽→ 왼쪽 출력 여부</param>
        /// <param name="minimumValue">최소 값</param>
        /// <param name="maximumValue">최대 값</param>
        /// <param name="value">값</param>
        private static void DrawProgressBar(Graphics graphics, Rectangle clientRectangle, Color backgroundColor, Color valueColor, Color borderColor, RightToLeft rightToLeft, int minimumValue, int maximumValue, int value)
        {
            Rectangle outerRectangle = GetBackgroundRectangle(clientRectangle);

            using(GraphicsPath outerRectangleGraphicsPath = GetBackgroundPath(outerRectangle, 4))
            {
                if(outerRectangleGraphicsPath != null)
                {
                    using(LinearGradientBrush brush = GetBackgroundBrush(outerRectangle, backgroundColor))
                    {
                        if(brush != null)
                        {
                            graphics.FillPath(brush, outerRectangleGraphicsPath);
                        }
                    }

                    if(value > 0)
                    {
                        Rectangle valueRectangle = GetValueRectangle(outerRectangle, rightToLeft, minimumValue, maximumValue, value);

                        using(GraphicsPath valueGraphicsPath = GetValuePath(valueRectangle, rightToLeft, 5))
                        {
                            using(LinearGradientBrush gradientBrush = GetBackgroundBrush(valueRectangle, valueColor))
                            {
                                if(gradientBrush != null)
                                {
                                    graphics.FillPath(gradientBrush, valueGraphicsPath);
                                }
                            }
                        }
                    }

                    using(Pen pen = new Pen(borderColor))
                    {
                        graphics.DrawPath(pen, outerRectangleGraphicsPath);
                    }
                }
            }
        }

        #endregion
    }
}