﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Windows.Forms.Design;

namespace TestProject
{
    /// <summary>
    /// 툴 스트립 진행 바
    /// </summary>
    [ToolStripItemDesignerAvailability(ToolStripItemDesignerAvailability.ToolStrip | ToolStripItemDesignerAvailability.StatusStrip)]
    [ToolboxBitmap(typeof(System.Windows.Forms.ProgressBar))]
    public class ToolStripProgressBar : ToolStripControlHost
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 진행 바 - ProgressBar

        /// <summary>
        /// 진행 바
        /// </summary>
        public ProgressBar ProgressBar
        {
            get
            {
                return base.Control as ProgressBar;
            }
        }

        #endregion
        #region 오른쪽→왼쪽 출력 여부 - RightToLeft

        /// <summary>
        /// 오른쪽→왼쪽 출력 여부
        /// </summary>
        public override RightToLeft RightToLeft
        {
            get
            {
                return ProgressBar.RightToLeft;
            }
            set
            {
                ProgressBar.RightToLeft = value;
            }
        }

        #endregion

        #region 배경 색상 - BackgroundColor

        /// <summary>
        /// 배경 색상
        /// </summary>
        [Category("ToolStripProgressBar")]
        [Browsable(true)]
        public Color BackgroundColor
        {
            get
            {
                return ProgressBar.BackgroundColor;
            }
            set
            {
                ProgressBar.BackgroundColor = value;
            }
        }

        #endregion
        #region 값 색상 - ValueColor

        /// <summary>
        /// 값 색상
        /// </summary>
        [Category("ToolStripProgressBar")]
        [Browsable(true)]
        public Color ValueColor
        {
            get
            {
                return ProgressBar.ValueColor;
            }
            set
            {
                ProgressBar.ValueColor = value;
            }
        }

        #endregion

        #region 전경 색상 - ForeColor

        /// <summary>
        /// 전경 색상
        /// </summary>
        [Category("ToolStripProgressBar")]
        [Browsable(true)]
        public override Color ForeColor
        {
            get
            {
                return ProgressBar.ForeColor;
            }
            set
            {
                ProgressBar.ForeColor = value;
            }
        }

        #endregion
        #region 폰트 - Font

        /// <summary>
        /// 폰트
        /// </summary>
        [Category("ToolStripProgressBar")]
        public override Font Font
        {
            get
            {
                return ProgressBar.Font;
            }
            set
            {
                ProgressBar.Font = value;
            }
        }

        #endregion

        #region 최소 값 - MinimumValue

        /// <summary>
        /// 최소 값
        /// </summary>
        [Category("ToolStripProgressBar")]
        [RefreshProperties(RefreshProperties.All)]
        [DefaultValue(0)]
        public int MinimumValue
        {
            get
            {
                return ProgressBar.MinimumValue;
            }
            set
            {
                ProgressBar.MinimumValue = value;
            }
        }

        #endregion
        #region 최대 값 - MaximumValue

        /// <summary>
        /// 최대 값
        /// </summary>
        [Category("ToolStripProgressBar")]
        [RefreshProperties(RefreshProperties.All)]
        [DefaultValue(100)]
        public int MaximumValue
        {
            get
            {
                return ProgressBar.MaximumValue;
            }
            set
            {
                ProgressBar.MaximumValue = value;
            }
        }

        #endregion
        #region 값 - Value

        /// <summary>
        /// 값
        /// </summary>
        [Category("ToolStripProgressBar")]
        [RefreshProperties(RefreshProperties.All)]
        [DefaultValue(0)]
        public int Value
        {
            get
            {
                return ProgressBar.Value;
            }
            set
            {
                ProgressBar.Value = value;
            }
        }

        #endregion

        #region 텍스트 - Text

        /// <summary>
        /// 텍스트
        /// </summary>
        [Category("ToolStripProgressBar")]
        public override string Text
        {
            get
            {
                return ProgressBar.Text;
            }
            set
            {
                ProgressBar.Text = value;
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 디폴트 크기 - DefaultSize

        /// <summary>
        /// 디폴트 크기
        /// </summary>
        protected override Size DefaultSize
        {
            get
            {
                return new Size(100, 15);
            }
        }

        #endregion
        #region 디폴트 마진 - DefaultMargin

        /// <summary>
        /// 디폴트 마진
        /// </summary>
        protected override Padding DefaultMargin
        {
            get
            {
                if((base.Owner != null) && (base.Owner is StatusStrip))
                {
                    return new Padding(1, 3, 1, 3);
                }

                return new Padding(1, 1, 1, 2);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ToolStripProgressBar()

        /// <summary>
        /// 생성자
        /// </summary>
        public ToolStripProgressBar() : base(CreateControl())
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 소유자 변경시 처리하기 - OnOwnerChanged(e)

        /// <summary>
        /// 소유자 변경시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnOwnerChanged(EventArgs e)
        {
            if(base.Owner != null)
            {
                base.Owner.RendererChanged += Owner_RendererChanged;
            }

            base.OnOwnerChanged(e);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 소유자 렌더러 변경시 처리하기 - Owner_RendererChanged(sender, e)

        /// <summary>
        /// 소유자 렌더러 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Owner_RendererChanged(object sender, EventArgs e)
        {
            ToolStripRenderer toolsTripRenderer = this.Owner.Renderer;

            if(toolsTripRenderer != null)
            {
                if(toolsTripRenderer is BaseRenderer)
                {
                    ToolStripProfessionalRenderer toolStripProfessionalRenderer = toolsTripRenderer as ToolStripProfessionalRenderer;

                    if(toolStripProfessionalRenderer != null)
                    {
                        ProgressBar.BorderColor = toolStripProfessionalRenderer.ColorTable.ToolStripBorder;
                    }

                    if(Owner.GetType() != typeof(StatusStrip))
                    {
                        Margin = new Padding(1, 1, 1, 3);
                    }
                }
                else
                {
                    Margin = DefaultMargin;
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 컨트롤 생성하기 - CreateControl()

        /// <summary>
        /// 컨트롤 생성하기
        /// </summary>
        /// <returns>컨트롤</returns>
        private static Control CreateControl()
        {
            ProgressBar progressBar = new ProgressBar();

            progressBar.Size = new Size(100, 15);

            return progressBar;
        }

        #endregion
    }
}