using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// ̽ 
    /// </summary>
    public class BaseRenderer : ToolStripProfessionalRenderer
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// ̽ ũ ׸ 簢 迭
        /// </summary>
        private static Rectangle[] _baseSizeGripRectangleArray;

        /// <summary>
        /// Թ 
        /// </summary>
        private static int _insetMargin;

        /// <summary>
        /// ޴ ׸ 
        /// </summary>
        private static Blend _menuItemBlend;

        /// <summary>
        /// ư 
        /// </summary>
        private static Blend _buttonBlend;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region  - BaseRenderer()

        /// <summary>
        /// 
        /// </summary>
		static BaseRenderer()
        {
            _insetMargin = 2;
            
            _buttonBlend = new Blend();

            _buttonBlend.Positions = new float[] { 0.0f, 0.1f, 0.2f, 0.5f, 1.0f };
            _buttonBlend.Factors   = new float[] { 0.6f, 0.7f, 0.8f, 1.0f, 1.0f };

            _menuItemBlend = new Blend();

            _menuItemBlend.Positions = new float[] { 0.0f, 0.1f, 0.2f, 0.5f, 1.0f };
            _menuItemBlend.Factors   = new float[] { 0.7f, 0.8f, 0.9f, 1.0f, 1.0f };

            _baseSizeGripRectangleArray = new Rectangle[]
            {
                new Rectangle(8, 0, 2, 2),
                new Rectangle(8, 4, 2, 2),
                new Rectangle(8, 8, 2, 2),
                new Rectangle(4, 4, 2, 2),
                new Rectangle(4, 8, 2, 2),
                new Rectangle(0, 8, 2, 2)
            };
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region  - BaseRenderer()

        /// <summary>
        /// 
        /// </summary>
        public BaseRenderer() : base(new BlackColorTable())
        {
			ColorTable.UseSystemColors = false;
		}

        #endregion
        #region  - BaseRenderer(professionalColorTable)

        /// <summary>
        /// 
        /// </summary>
        /// <param name="professionalColorTable">ų  ̺</param>
        public BaseRenderer(ProfessionalColorTable professionalColorTable) : base(professionalColorTable)
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        //////////////////////////////////////////////////////////////////////////////// Private

        #region   ϱ - GetBackgroundPath(boundRectangle, radius)

        /// <summary>
        ///   ϱ
        /// </summary>
        /// <param name="boundRectangle"> 簢</param>
        /// <param name="radius"></param>
        /// <returns> </returns>
        private static GraphicsPath GetBackgroundPath(Rectangle boundRectangle, int radius)
        {
            int x      = boundRectangle.X;
            int y      = boundRectangle.Y;
            int width  = boundRectangle.Width;
            int height = boundRectangle.Height;

            GraphicsPath graphicsPath = new GraphicsPath();

            graphicsPath.AddArc(x                 , y                  , radius, radius, 180, 90);
            graphicsPath.AddArc(x + width - radius, y                  , radius, radius, 270, 90);
            graphicsPath.AddArc(x + width - radius, y + height - radius, radius, radius, 0  , 90);
            graphicsPath.AddArc(x                 , y + height - radius, radius, radius, 90 , 90);

            graphicsPath.CloseFigure();

            return graphicsPath;
        }

        #endregion
        #region ư 簢 ϱ - GetButtonRectangle(boundRectangle)

        /// <summary>
        /// ư 簢 ϱ
        /// </summary>
        /// <param name="boundRectangle"> 簢</param>
        /// <returns>ư 簢</returns>
        private static Rectangle GetButtonRectangle(Rectangle boundRectangle)
        {
            Rectangle rectangle = boundRectangle;

            rectangle.Width  -= 1;
            rectangle.Height -= 1;

            rectangle.Inflate(0, -1);

            return rectangle;
        }

        #endregion

        #region ʺ Ǵ  0  ϱ - IsZeroWidthOrHeight(rectangle)

        /// <summary>
        /// ʺ Ǵ  0  ϱ
        /// </summary>
        /// <param name="rectangle">簢</param>
        /// <returns>ʺ Ǵ  0 </returns>
        private static bool IsZeroWidthOrHeight(Rectangle rectangle)
        {
            if(rectangle.Width != 0)
            {
                return (rectangle.Height == 0);
            }

            return true;
        }

        #endregion

        #region ư ϱ - RenderButton(graphics, boundRectangle, color)

        /// <summary>
        /// ư ϱ
        /// </summary>
        /// <param name="graphics">׷Ƚ</param>
        /// <param name="boundRectangle"> 簢</param>
        /// <param name="color"></param>
        private static void RenderButton(Graphics graphics, Rectangle boundRectangle, Color color)
        {
            using(GraphicsPath graphicsPath = GetBackgroundPath(boundRectangle, 3))
            {
                using
                (
                    LinearGradientBrush brush = new LinearGradientBrush
                    (
                        boundRectangle,
                        color,
                        Color.Transparent,
                        LinearGradientMode.Vertical
                    )
                )
                {
                    brush.Blend = _buttonBlend;

                    graphics.FillPath(brush, graphicsPath);
                }
            }
        }

        #endregion
        #region ޴ ׸ ϱ - RenderMenuItem(graphics, boundRectangle, color)

        /// <summary>
        /// ޴ ׸ ϱ
        /// </summary>
        /// <param name="graphics">׷Ƚ</param>
        /// <param name="boundRectangle"> 簢</param>
        /// <param name="color"></param>
        private static void RenderMenuItem(Graphics graphics, Rectangle boundRectangle, Color color)
        {
            using
            (
                LinearGradientBrush brush = new LinearGradientBrush
                (
                    boundRectangle,
                    color,
                    Color.Transparent,
                    LinearGradientMode.Vertical
                )
            )
            {
                brush.Blend = _menuItemBlend;

                graphics.FillRectangle(brush, boundRectangle);
            }
        }

        #endregion
        #region ȭǥ ϱ (ο) - RenderArrowInternal(graphics, dropDownRectangle, direction, color)

        /// <summary>
        /// ȭǥ ϱ (ο)
        /// </summary>
        /// <param name="graphics">׷Ƚ</param>
        /// <param name="dropDownRectangle">DROP DOWN 簢</param>
        /// <param name="direction"></param>
        /// <param name="color"></param>
        private static void RenderArrowInternal(Graphics graphics, Rectangle dropDownRectangle, ArrowDirection direction, Color color)
        {
            Point point = new Point(dropDownRectangle.Left + (dropDownRectangle.Width / 2), dropDownRectangle.Top + (dropDownRectangle.Height / 2));

            point.X += dropDownRectangle.Width % 2;

            Point[] pointArray = null;

            switch(direction)
            {
                case ArrowDirection.Left :

                    pointArray = new Point[] { new Point(point.X + 2, point.Y - 3), new Point(point.X + 2, point.Y + 3), new Point(point.X - 1, point.Y) };

                    break;

                case ArrowDirection.Up :

                    pointArray = new Point[] { new Point(point.X - 2, point.Y + 1), new Point(point.X + 3, point.Y + 1), new Point(point.X, point.Y - 2) };

                    break;

                case ArrowDirection.Right :

                    pointArray = new Point[] { new Point(point.X - 2, point.Y - 3), new Point(point.X - 2, point.Y + 3), new Point(point.X + 1, point.Y) };

                    break;

                default :

                    pointArray = new Point[] { new Point(point.X - 2, point.Y - 1), new Point(point.X + 3, point.Y - 1), new Point(point.X, point.Y + 2) };

                    break;
            }

            using(SolidBrush brush = new SolidBrush(color))
            {
                graphics.FillPolygon(brush, pointArray);
            }
        }

        #endregion
        #region ÷ ư   ϱ - RenderOverflowButtonLine(graphics, color, x1, y1, x2, y2)

        /// <summary>
        /// ÷ ư   ϱ
        /// </summary>
        /// <param name="graphics">׷Ƚ</param>
        /// <param name="color"></param>
        /// <param name="x1">X1 ǥ</param>
        /// <param name="y1">Y1 ǥ</param>
        /// <param name="x2">X2 ǥ</param>
        /// <param name="y2">Y2 ǥ</param>
        private static void RenderOverflowButtonLine(Graphics graphics, Color color, int x1, int y1, int x2, int y2)
        {
            using(Pen pen = new Pen(color))
            {
                graphics.DrawLine(pen, x1, y1, x2, y2);
            }
        }

        #endregion

        #region ư ׵θ ׸ - DrawButtonBorder(graphics, boundRectangle, color)

        /// <summary>
        /// ư ׵θ ׸
        /// </summary>
        /// <param name="graphics">׷Ƚ</param>
        /// <param name="boundRectangle"> 簢</param>
        /// <param name="color"></param>
        private static void DrawButtonBorder(Graphics graphics, Rectangle boundRectangle, Color color)
        {
            using(GraphicsPath graphicsPath = GetBackgroundPath(boundRectangle, 3))
            {
                using(Pen pen = new Pen(color))
                {
                    graphics.DrawPath(pen, graphicsPath);
                }
            }
        }

        #endregion
        #region ư  ׵θ ׸ - DrawButtonInnerBorder(graphics, boundRectangle, color)

        /// <summary>
        /// ư  ׵θ ׸
        /// </summary>
        /// <param name="graphics">׷Ƚ</param>
        /// <param name="boundRectangle"> 簢</param>
        /// <param name="color"></param>
        private static void DrawButtonInnerBorder(Graphics graphics, Rectangle boundRectangle, Color color)
        {
            Rectangle innerBoundRectangle = boundRectangle;

            innerBoundRectangle.Height -= 1;
            innerBoundRectangle.Width  -= 1;

            using(GraphicsPath graphicsPath = GetBackgroundPath(innerBoundRectangle, 3))
            {
                using(Pen pen = new Pen(color))
                {
                    graphics.DrawPath(pen, graphicsPath);
                }
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Protected

        #region ȭǥ  óϱ - OnRenderArrow(e)

        /// <summary>
        /// ȭǥ  óϱ
        /// </summary>
        /// <param name="e">̺Ʈ </param>
        protected override void OnRenderArrow(ToolStripArrowRenderEventArgs e)
        {
            if(ColorTable.UseSystemColors == true)
            {
                base.OnRenderArrow(e);
            }
            else
            {
                ProfessionalColorTable colorTable = ColorTable as ProfessionalColorTable;

                if((colorTable != null) && (e.Item.Enabled == true))
                {
                    if(e.Item.Owner is MenuStrip)
                    {
                        e.ArrowColor = colorTable.MenuItemText;
                    }
                    else if(e.Item.Owner is StatusStrip)
                    {
                        e.ArrowColor = colorTable.StatusStripText;
                    }
                    else
                    {
                        if(e.Item.Owner.GetType() != typeof(ToolStripDropDownMenu))
                        {
                            e.ArrowColor = colorTable.ToolStripText;
                        }
                    }
                }

                base.OnRenderArrow(e);
            }
        }

        #endregion
        #region ư   óϱ - OnRenderButtonBackground(e)

        /// <summary>
        /// ư   óϱ
        /// </summary>
        /// <param name="e">̺Ʈ </param>
        protected override void OnRenderButtonBackground(ToolStripItemRenderEventArgs e)
        {
            if(ColorTable.UseSystemColors == true)
            {
                base.OnRenderDropDownButtonBackground(e);
            }
            else
            {
                ToolStripButton toolStripButton = e.Item as ToolStripButton;

                Rectangle buttonBoundRectangle = new Rectangle(Point.Empty, toolStripButton.Size);

                if(IsZeroWidthOrHeight(buttonBoundRectangle) == true)
                {
                    return;
                }

                Graphics graphics = e.Graphics;

                ProfessionalColorTable colorTable = ColorTable as ProfessionalColorTable;

                if(colorTable != null)
                {
                    using(AntiAliasHelper antiAlias = new AntiAliasHelper(graphics))
                    {
                        Rectangle buttonRectangle = GetButtonRectangle(buttonBoundRectangle);

                        if(toolStripButton.Checked == true)
                        {
                            DrawButtonBorder(graphics, buttonRectangle, colorTable.ButtonPressedBorder);
                        }

                        if((toolStripButton.Selected == true) && (toolStripButton.Pressed == false))
                        {
                            RenderButton(graphics, buttonRectangle, colorTable.MenuItemTopLevelSelectedGradientBegin);

                            DrawButtonBorder(graphics, buttonRectangle, colorTable.ButtonSelectedHighlightBorder);
                        }

                        if(toolStripButton.Pressed == true)
                        {
                            RenderButton(graphics, buttonRectangle, colorTable.MenuItemPressedGradientBegin);

                            DrawButtonInnerBorder(graphics, buttonRectangle, colorTable.ButtonSelectedHighlightBorder);

                            DrawButtonBorder(graphics, buttonRectangle, colorTable.MenuBorder);
                        }
                    }
                }
                else
                {
                    base.OnRenderDropDownButtonBackground(e);
                }
            }
        }

        #endregion
        #region DROP DOWN ư   óϱ - OnRenderDropDownButtonBackground(e)

        /// <summary>
        /// DROP DOWN ư   óϱ
        /// </summary>
        /// <param name="e">̺Ʈ </param>
        protected override void OnRenderDropDownButtonBackground(ToolStripItemRenderEventArgs e)
        {
            if(ColorTable.UseSystemColors == true)
            {
                base.OnRenderDropDownButtonBackground(e);
            }
            else
            {
                ToolStripDropDownButton toolStripDropDownButton = e.Item as ToolStripDropDownButton;

                Rectangle buttonBoundRectangle = new Rectangle(Point.Empty, toolStripDropDownButton.Size);

                if(IsZeroWidthOrHeight(buttonBoundRectangle) == true)
                {
                    return;
                }

                Graphics graphics = e.Graphics;

                ProfessionalColorTable colorTable = ColorTable as ProfessionalColorTable;

                if(colorTable != null)
                {
                    using(AntiAliasHelper antiAlias = new AntiAliasHelper(graphics))
                    {
                        Rectangle buttonRectangle = GetButtonRectangle(buttonBoundRectangle);

                        if((toolStripDropDownButton.Selected == true) && (toolStripDropDownButton.Pressed == false))
                        {
                            RenderButton(graphics, buttonRectangle, colorTable.MenuItemTopLevelSelectedGradientBegin);

                            DrawButtonBorder(graphics, buttonRectangle, colorTable.ButtonSelectedHighlightBorder);
                        }

                        if(toolStripDropDownButton.Pressed == true)
                        {
                            RenderButton(graphics, buttonRectangle, colorTable.MenuItemPressedGradientBegin);

                            DrawButtonInnerBorder(graphics, buttonRectangle, colorTable.ButtonSelectedHighlightBorder);

                            DrawButtonBorder(graphics, buttonRectangle, colorTable.MenuBorder);
                        }
                    }
                }
                else
                {
                    base.OnRenderDropDownButtonBackground(e);
                }
            }
        }

        #endregion
        #region ø ư   óϱ - OnRenderSplitButtonBackground(e)

        /// <summary>
        /// ø ư   óϱ
        /// </summary>
        /// <param name="e">̺Ʈ </param>
        protected override void OnRenderSplitButtonBackground(ToolStripItemRenderEventArgs e)
        {
            if(ColorTable.UseSystemColors == true)
            {
                base.OnRenderDropDownButtonBackground(e);
            }
            else
            {
                ToolStripSplitButton toolStripSplitButton = e.Item as ToolStripSplitButton;

                Rectangle buttonBoundRectangle = new Rectangle(Point.Empty, toolStripSplitButton.ButtonBounds.Size);

                if(IsZeroWidthOrHeight(buttonBoundRectangle) == true)
                {
                    return;
                }

                Graphics graphics = e.Graphics;

                ProfessionalColorTable colorTable = ColorTable as ProfessionalColorTable;

                if(colorTable != null)
                {
                    using(AntiAliasHelper antiAlias = new AntiAliasHelper(graphics))
                    {
                        Rectangle buttonRectangle = GetButtonRectangle(buttonBoundRectangle);

                        Rectangle dropDownButtonBoundRectangle = new Rectangle(toolStripSplitButton.DropDownButtonBounds.Location, toolStripSplitButton.DropDownButtonBounds.Size);

                        Rectangle dropDownButtonRectangle = GetButtonRectangle(dropDownButtonBoundRectangle);

                        if((toolStripSplitButton.Selected == true) && (toolStripSplitButton.Pressed == false) && (toolStripSplitButton.ButtonPressed == false))
                        {
                            RenderButton(graphics, buttonRectangle, colorTable.MenuItemTopLevelSelectedGradientBegin);

                            RenderButton(graphics, dropDownButtonRectangle, colorTable.MenuItemTopLevelSelectedGradientBegin);

                            DrawButtonBorder(graphics, buttonRectangle, colorTable.ButtonSelectedHighlightBorder);

                            DrawButtonBorder(graphics, dropDownButtonRectangle, colorTable.ButtonSelectedHighlightBorder);
                        }

                        if(toolStripSplitButton.ButtonPressed == true)
                        {
                            RenderButton(graphics, buttonRectangle, colorTable.MenuItemPressedGradientBegin);

                            RenderButton(graphics, dropDownButtonRectangle, colorTable.MenuItemPressedGradientBegin);

                            DrawButtonInnerBorder(graphics, buttonRectangle, colorTable.ButtonSelectedHighlightBorder);

                            DrawButtonBorder(graphics, buttonRectangle, colorTable.MenuBorder);

                            DrawButtonInnerBorder(graphics, dropDownButtonRectangle, colorTable.ButtonSelectedHighlightBorder);

                            DrawButtonBorder(graphics, dropDownButtonRectangle, colorTable.MenuBorder);
                        }

                        if(toolStripSplitButton.DropDownButtonPressed == true)
                        {
                            RenderButton(graphics, buttonRectangle, colorTable.MenuItemTopLevelSelectedGradientBegin);

                            RenderButton(graphics, dropDownButtonRectangle, colorTable.MenuItemTopLevelSelectedGradientBegin);

                            DrawButtonBorder(graphics, buttonRectangle, ColorTable.ButtonSelectedHighlightBorder);

                            DrawButtonBorder(graphics, dropDownButtonRectangle, ColorTable.ButtonSelectedHighlightBorder);
                        }

                        if(e.Item.Owner is MenuStrip)
                        {
                            base.DrawArrow(new ToolStripArrowRenderEventArgs(graphics, toolStripSplitButton, dropDownButtonBoundRectangle, colorTable.MenuItemText, ArrowDirection.Down));
                        }

                        if(e.Item.Owner is StatusStrip)
                        {
                            base.DrawArrow(new ToolStripArrowRenderEventArgs(graphics, toolStripSplitButton, dropDownButtonBoundRectangle, colorTable.StatusStripText, ArrowDirection.Down));
                        }

                        if(e.Item.Owner is ToolStrip)
                        {
                            base.DrawArrow(new ToolStripArrowRenderEventArgs(graphics, toolStripSplitButton, dropDownButtonBoundRectangle, colorTable.ToolStripText, ArrowDirection.Down));
                        }
                    }
                }
                else
                {
                    base.OnRenderDropDownButtonBackground(e);
                }
            }
        }

        #endregion
        #region ޴ ׸   óϱ - OnRenderMenuItemBackground(e)

        /// <summary>
        /// ޴ ׸   óϱ
        /// </summary>
        /// <param name="e">̺Ʈ </param>
        protected override void OnRenderMenuItemBackground(ToolStripItemRenderEventArgs e)
        {
            ToolStripMenuItem toolStripMenuItem = e.Item as ToolStripMenuItem;

            Rectangle boundRectangle = new Rectangle(Point.Empty, toolStripMenuItem.Size);

            if(IsZeroWidthOrHeight(boundRectangle) == true)
            {
                return;
            }

            Graphics graphics = e.Graphics;

            ProfessionalColorTable colorTable = ColorTable as TestProject.ProfessionalColorTable;

            if(colorTable != null)
            {
                using(AntiAliasHelper helper = new AntiAliasHelper(graphics))
                {
                    if(e.ToolStrip is MenuStrip)
                    {
                        if((toolStripMenuItem.Selected == true) && (toolStripMenuItem.Pressed == false))
                        {
                            RenderMenuItem(graphics, boundRectangle, colorTable.MenuItemTopLevelSelectedGradientBegin);

                            ControlPaint.DrawBorder(e.Graphics, boundRectangle, colorTable.MenuItemTopLevelSelectedBorder, ButtonBorderStyle.Solid);
                        }

                        if(toolStripMenuItem.Pressed == true)
                        {
                            RenderButton(graphics, boundRectangle, ColorTable.MenuItemPressedGradientBegin);

                            Rectangle innerBorderRectangle = boundRectangle;

                            innerBorderRectangle.Inflate(-1, -1);

                            ControlPaint.DrawBorder(e.Graphics, innerBorderRectangle, ColorTable.ButtonSelectedHighlightBorder, ButtonBorderStyle.Solid);

                            ControlPaint.DrawBorder(e.Graphics, boundRectangle, ColorTable.MenuBorder, ButtonBorderStyle.Solid);
                        }
                    }
                    else
                    {
                        base.OnRenderMenuItemBackground(e);
                    }
                }
            }
            else
            {
                base.OnRenderMenuItemBackground(e);
            }
        }

        #endregion
        #region ׸ ؽƮ  óϱ - OnRenderItemText(e)

        /// <summary>
        /// ׸ ؽƮ  óϱ
        /// </summary>
        /// <param name="e">̺Ʈ </param>
        protected override void OnRenderItemText(ToolStripItemTextRenderEventArgs e)
        {
			if(ColorTable.UseSystemColors == false)
			{
				ProfessionalColorTable colorTable = ColorTable as ProfessionalColorTable;

				if(colorTable != null)
				{
                    if((e.ToolStrip is MenuStrip))
                    {
                        if(colorTable.MenuItemText != Color.Empty)
                        {
                            e.TextColor = colorTable.MenuItemText;
                        }
                    }
                    else if((e.ToolStrip is StatusStrip))
                    {
                        if(colorTable.StatusStripText != Color.Empty)
                        {
                            e.TextColor = colorTable.StatusStripText;
                        }
                    }
                    else if(e.ToolStrip is ToolStripDropDown)
                    {
                    }
                    else
                    {
                        if(colorTable.ToolStripText != Color.Empty)
                        {
                            e.TextColor = colorTable.ToolStripText;
                        }
                    }
				}
			}

            base.OnRenderItemText(e);
        }

        #endregion
        #region  Ʈ Ʈ г   óϱ - OnRenderToolStripContentPanelBackground(e)

        /// <summary>
        ///  Ʈ Ʈ г   óϱ
        /// </summary>
        /// <param name="e">̺Ʈ </param>
        protected override void OnRenderToolStripContentPanelBackground(ToolStripContentPanelRenderEventArgs e)
        {
            base.OnRenderToolStripContentPanelBackground(e);

			if(ColorTable.UseSystemColors == false)
			{
				if((e.ToolStripContentPanel.Width > 0) && (e.ToolStripContentPanel.Height > 0))
				{
					using
                    (
                        LinearGradientBrush brush = new LinearGradientBrush
                        (
                            e.ToolStripContentPanel.ClientRectangle,
							ColorTable.ToolStripContentPanelGradientBegin,
							ColorTable.ToolStripContentPanelGradientEnd,
							LinearGradientMode.Vertical
                        )
                    )
					{
						e.Graphics.FillRectangle(brush, e.ToolStripContentPanel.ClientRectangle);
					}
				}
			}
        }

        #endregion
        #region ÷ο ư   óϱ - OnRenderOverflowButtonBackground(e)

        /// <summary>
        /// ÷ο ư   óϱ
        /// </summary>
        /// <param name="e">̺Ʈ </param>
        protected override void OnRenderOverflowButtonBackground(ToolStripItemRenderEventArgs e)
        {
            base.OnRenderOverflowButtonBackground(e);

            ToolStripItem toolStripItem = e.Item;

            if((toolStripItem.Selected == false) && (toolStripItem.Pressed == false))
            {
                ProfessionalColorTable colorTable = ColorTable as ProfessionalColorTable;

                if(colorTable != null)
                {
                    Graphics graphics = e.Graphics;

                    bool isRightToLeft = toolStripItem.RightToLeft == RightToLeft.Yes;

                    bool isHorizontal = e.ToolStrip.Orientation == Orientation.Horizontal;

                    Rectangle arrowRectangle = Rectangle.Empty;

                    if(isRightToLeft)
                    {
                        arrowRectangle = new Rectangle(0, toolStripItem.Height - 8, 9, 5);
                    }
                    else
                    {
                        arrowRectangle = new Rectangle(toolStripItem.Width - 12, toolStripItem.Height - 8, 9, 5);
                    }

                    ArrowDirection arrowDirection = isHorizontal ? ArrowDirection.Down : ArrowDirection.Right;

                    int x = (isRightToLeft && isHorizontal) ? -1 : 1;

                    arrowRectangle.Offset(x, 1);

                    RenderArrowInternal(graphics, arrowRectangle, arrowDirection, colorTable.ToolStripGradientMiddle);

                    arrowRectangle.Offset(-1 * x, -1);

                    RenderArrowInternal(graphics, arrowRectangle, arrowDirection, colorTable.ToolStripText);

                    if(isHorizontal)
                    {
                        x = isRightToLeft ? -2 : 0;

                        RenderOverflowButtonLine(graphics, colorTable.ToolStripText, (int)(arrowRectangle.Right - 6), (int)(arrowRectangle.Y - 2), (int)(arrowRectangle.Right - 2), (int)(arrowRectangle.Y - 2));

                        RenderOverflowButtonLine(graphics, colorTable.ToolStripGradientMiddle, (int)((arrowRectangle.Right - 5) + x), (int)(arrowRectangle.Y - 1), (int)((arrowRectangle.Right - 1) + x), (int)(arrowRectangle.Y - 1));
                    }
                    else
                    {
                        RenderOverflowButtonLine(graphics, colorTable.ToolStripText, arrowRectangle.X, arrowRectangle.Y, arrowRectangle.X, arrowRectangle.Bottom - 1);

                        RenderOverflowButtonLine(graphics, colorTable.ToolStripGradientMiddle, arrowRectangle.X + 1, arrowRectangle.Y + 1, arrowRectangle.X + 1, arrowRectangle.Bottom);
                    }
                }
            }
        }

        #endregion
        #region и  óϱ - OnRenderSeparator(e)

        /// <summary>
        /// и  óϱ
        /// </summary>
        /// <param name="e">̺Ʈ </param>
        protected override void OnRenderSeparator(ToolStripSeparatorRenderEventArgs e)
        {
			if(ColorTable.UseSystemColors == false)
			{
				e.Item.ForeColor = ColorTable.RaftingContainerGradientBegin;
			}

            base.OnRenderSeparator(e);
        }

        #endregion
        #region  Ʈ ũ  ׸  óϱ - OnRenderStatusStripSizingGrip(e)

        /// <summary>
        ///  Ʈ ũ  ׸  óϱ
        /// </summary>
        /// <param name="e">̺Ʈ </param>
        protected override void OnRenderStatusStripSizingGrip(ToolStripRenderEventArgs e)
        {
            Graphics graphics = e.Graphics;

            StatusStrip statusStrip = e.ToolStrip as StatusStrip;

            if(statusStrip != null)
            {
                Rectangle sizeGripBoundRectangle = statusStrip.SizeGripBounds;

                if(IsZeroWidthOrHeight(sizeGripBoundRectangle) == false)
                {
                    Rectangle[] lightRectangleArray = new Rectangle[_baseSizeGripRectangleArray.Length];
                    Rectangle[] darkRectangleArray  = new Rectangle[_baseSizeGripRectangleArray.Length];

                    for(int i = 0; i < _baseSizeGripRectangleArray.Length; i++)
                    {
                        Rectangle darkRectangle = _baseSizeGripRectangleArray[i];

                        if(statusStrip.RightToLeft == RightToLeft.Yes)
                        {
                            darkRectangle.X = (sizeGripBoundRectangle.Width - darkRectangle.X) - darkRectangle.Width;
                        }

                        darkRectangle.Offset(sizeGripBoundRectangle.X, sizeGripBoundRectangle.Bottom - 12);

                        lightRectangleArray[i] = darkRectangle;

                        if(statusStrip.RightToLeft == RightToLeft.Yes)
                        {
                            darkRectangle.Offset(1, -1);
                        }
                        else
                        {
                            darkRectangle.Offset(-1, -1);
                        }

                        darkRectangleArray[i] = darkRectangle;
                    }

                    using(SolidBrush brush = new SolidBrush(ColorTable.GripDark), lightBrush = new SolidBrush(ColorTable.GripDark))
                    {
                        graphics.FillRectangles(lightBrush, lightRectangleArray);

                        graphics.FillRectangles(brush, darkRectangleArray);
                    }
                }
            }
        }

        #endregion
        #region  Ʈ   óϱ - OnRenderToolStripBackground(e)

        /// <summary>
        ///  Ʈ   óϱ
        /// </summary>
        /// <param name="e">̺Ʈ </param>
        protected override void OnRenderToolStripBackground(ToolStripRenderEventArgs e)
        {
            if(ColorTable.UseSystemColors == true)
            {
                base.OnRenderToolStripBackground(e);
            }
            else
            {
                Rectangle backgroundRectangle = new Rectangle(0, 0, e.ToolStrip.Width, e.ToolStrip.Height);

                Rectangle innerRectangle = backgroundRectangle;

                innerRectangle.Height = (backgroundRectangle.Height / 2) + 1;

                if((backgroundRectangle.Width > 0) && (backgroundRectangle.Height > 0))
                {
                    if(e.ToolStrip is StatusStrip)
                    {
                        using(SolidBrush outerBrush = new SolidBrush(ColorTable.StatusStripGradientEnd))
                        {
                            e.Graphics.FillRectangle(outerBrush, backgroundRectangle);
                        }

                        int y2 = backgroundRectangle.Height / 2;

                        Rectangle upperRectangle = new Rectangle(backgroundRectangle.X, backgroundRectangle.Y, backgroundRectangle.Width, y2);

                        upperRectangle.Height += 1;

                        using
                        (
                            LinearGradientBrush brush = new LinearGradientBrush
                            (
                                upperRectangle,
                                ColorTable.StatusStripGradientBegin,
                                Color.FromArgb(128, ColorTable.StatusStripGradientBegin),
                                LinearGradientMode.Vertical
                            )
                        )
                        {
                            e.Graphics.FillRectangle(brush, upperRectangle);
                        }

                        y2 = (backgroundRectangle.Height / 4) + 1;

                        Rectangle lowerRectangle = new Rectangle(backgroundRectangle.X, backgroundRectangle.Height - y2, backgroundRectangle.Width, y2);

                        using
                        (
                            LinearGradientBrush brush = new LinearGradientBrush
                            (
                                lowerRectangle,
                                ColorTable.StatusStripGradientEnd,
                                Color.FromArgb(128, ColorTable.StatusStripGradientBegin),
                                LinearGradientMode.Vertical
                            )
                        )
                        {
                            e.Graphics.FillRectangle(brush, lowerRectangle);
                        }
                    }
                    else if(e.ToolStrip is MenuStrip)
                    {
                        using(SolidBrush brush = new SolidBrush(ColorTable.MenuStripGradientEnd))
                        {
                            e.Graphics.FillRectangle(brush, backgroundRectangle);
                        }

                        int y2 = backgroundRectangle.Height / 3;

                        Rectangle lowerRectangle = new Rectangle(backgroundRectangle.X, backgroundRectangle.Y, backgroundRectangle.Width, y2);

                        using
                        (
                            LinearGradientBrush brush = new LinearGradientBrush
                            (
                                lowerRectangle,
                                ColorTable.MenuStripGradientBegin,
                                Color.FromArgb(128, ColorTable.StatusStripGradientBegin),
                                LinearGradientMode.Vertical
                            )
                        )
                        {
                            e.Graphics.FillRectangle(brush, lowerRectangle);
                        }
                    }
                    else if(e.ToolStrip is ToolStripDropDown)
                    {
                        base.OnRenderToolStripBackground(e);
                    }
                    else
                    {
                        using(SolidBrush outerBrush = new SolidBrush(ColorTable.ToolStripGradientEnd))
                        {
                            e.Graphics.FillRectangle(outerBrush, backgroundRectangle);
                        }

                        int y2 = backgroundRectangle.Height / 2;

                        Rectangle upperRectangle = new Rectangle(backgroundRectangle.X, backgroundRectangle.Y, backgroundRectangle.Width, y2);

                        using
                        (
                            LinearGradientBrush brush = new LinearGradientBrush
                            (
                                upperRectangle,
                                ColorTable.ToolStripGradientBegin,
                                ColorTable.ToolStripGradientMiddle,
                                LinearGradientMode.Vertical
                            )
                        )
                        {
                            e.Graphics.FillRectangle(brush, upperRectangle);
                        }

                        y2 = backgroundRectangle.Height / 4;

                        Rectangle lowerRectangle = new Rectangle(backgroundRectangle.X, backgroundRectangle.Height - y2, backgroundRectangle.Width, y2);

                        using
                        (
                            LinearGradientBrush brush = new LinearGradientBrush
                            (
                                lowerRectangle,
                                ColorTable.ToolStripGradientEnd,
                                ColorTable.ToolStripGradientMiddle,
                                LinearGradientMode.Vertical
                            )
                        )
                        {
                            e.Graphics.FillRectangle(brush, lowerRectangle);
                        }
                    }
                }
            }
        }

        #endregion
        #region ̹   óϱ - OnRenderImageMargin(e)

        /// <summary>
        /// ̹   óϱ
        /// </summary>
        /// <param name="e">̺Ʈ </param>
        protected override void OnRenderImageMargin(ToolStripRenderEventArgs e)
        {
			if(ColorTable.UseSystemColors == true)
			{
				base.OnRenderToolStripBackground(e);
			}
			else
			{
				if((e.ToolStrip is ContextMenuStrip) || (e.ToolStrip is ToolStripDropDownMenu))
				{
					Rectangle marginRectangle = e.AffectedBounds;

					bool isRightToLeft = (e.ToolStrip.RightToLeft == RightToLeft.Yes);

					marginRectangle.Y      += _insetMargin;
					marginRectangle.Height -= _insetMargin * 2;

					if(isRightToLeft == false)
					{
						marginRectangle.X += _insetMargin;
					}
					else
					{
						marginRectangle.X += _insetMargin / 2;
					}

					using(SolidBrush brush = new SolidBrush(ColorTable.ImageMarginGradientBegin))
                    {
						e.Graphics.FillRectangle(brush, marginRectangle);
                    }
				}
				else
				{
					base.OnRenderImageMargin(e);
				}
			}
        }

		#endregion
    }
}